"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolve = exports.PreprocessorConfiguration = void 0;
const cosmiconfig_1 = require("cosmiconfig");
const util_1 = __importDefault(require("util"));
const debug_1 = __importDefault(require("./debug"));
const type_guards_1 = require("./type-guards");
function validateConfigurationEntry(key, value) {
    switch (key) {
        case "globalStepDefinitions":
            if (!type_guards_1.isBoolean(value)) {
                throw new Error(`Expected a string (globalStepDefinitions), but got ${util_1.default.inspect(value)}`);
            }
            return { [key]: value };
        case "stepDefinitionsFolder":
            if (!type_guards_1.isString(value)) {
                throw new Error(`Expected a string (stepDefinitionsFolder), but got ${util_1.default.inspect(value)}`);
            }
            return { [key]: value };
        case "stepDefinitionsCommonFolder":
            if (!type_guards_1.isString(value)) {
                throw new Error(`Expected a string (stepDefinitionsCommonFolder), but got ${util_1.default.inspect(value)}`);
            }
            return { [key]: value };
        case "integrationFolder":
            if (!type_guards_1.isString(value)) {
                throw new Error(`Expected a string (integrationFolder), but got ${util_1.default.inspect(value)}`);
            }
            return { [key]: value };
        default:
            return {};
    }
}
class PreprocessorConfiguration {
    constructor(explicitValues) {
        this.explicitValues = explicitValues;
    }
    get globalStepDefinitions() {
        if (this.explicitValues.globalStepDefinitions === false) {
            return false;
        }
        return this.explicitValues.globalStepDefinitions || true;
    }
    get stepDefinitionsFolder() {
        if (this.explicitValues.stepDefinitionsFolder) {
            return this.explicitValues.stepDefinitionsFolder;
        }
        else if (this.globalStepDefinitions) {
            return "cypress/support/step_definitions";
        }
        else {
            return this.integrationFolder;
        }
    }
    get stepDefinitionsCommonFolder() {
        return this.explicitValues.stepDefinitionsCommonFolder || "common";
    }
    get integrationFolder() {
        return this.explicitValues.integrationFolder || "cypress/integration";
    }
}
exports.PreprocessorConfiguration = PreprocessorConfiguration;
async function resolve() {
    const result = await cosmiconfig_1.cosmiconfig("cypress-cucumber-preprocessor").search();
    if (result) {
        const { config: rawConfig } = result;
        if (typeof rawConfig !== "object" || rawConfig == null) {
            throw new Error(`Malformed configuration, expected an object, but got ${util_1.default.inspect(rawConfig)}`);
        }
        const config = Object.assign({}, ...Object.entries(rawConfig).map((entry) => validateConfigurationEntry(...entry)));
        debug_1.default(`resolved configuration ${util_1.default.inspect(config)}`);
        return new PreprocessorConfiguration(config);
    }
    else {
        debug_1.default("resolved no configuration");
        return new PreprocessorConfiguration({});
    }
}
exports.resolve = resolve;
